/* $Novell: /cm/ldap/src/cldap/samples/getdse.c,v 1.12 2002/01/09 02:03:41 dsward Exp $ */
/*************************************************************************

   Copyright 1999, 2000, 2001 Novell, Inc.  All Rights Reserved.

   With respect to this file, Novell hereby grants to Developer a 
   royalty-free, non-exclusive license to include this sample code 
   and derivative binaries in its product. Novell grants to Developer 
   worldwide distribution rights to market, distribute or sell this 
   sample code file and derivative binaries as a component of 
   Developer's product(s).  Novell shall have no obligations to 
   Developer or Developer's customers with respect to this code.

   DISCLAIMER:

   Novell disclaims and excludes any and all express, implied, and 
   statutory warranties, including, without limitation, warranties 
   of good title, warranties against infringement, and the implied 
   warranties of merchantability and fitness for a particular purpose.  
   Novell does not warrant that the software will satisfy customer's 
   requirements or that the licensed works are without defect or error 
   or that the operation of the software will be uninterrupted.  
   Novell makes no warranties respecting any technical services or 
   support tools provided under the agreement, and disclaims all other 
   warranties, including the implied warranties of merchantability and 
   fitness for a particular purpose.

***************************************************************************
   getdse.c 
***************************************************************************
   Description: The getdse.c sample retrieves the root DSE of the LDAP server
                specified by the LDAP session handle and displays the attribute
                values. For known extensions and controls, a lookup table is 
				used to print a descriptive name.
 
   Sample Run:   Root DSE
                 -----------------------------------------
                 supportedSASLMechanisms:
                     EXTERNAL
                 supportedLDAPVersion:
                     2
                     3
                 supportedExtension:
                     2.16.840.1.113719.1.27.100.55 (TRIGGER_PART_PURGE_REQ)
                     2.16.840.1.113719.1.27.100.56 (TRIGGER_PART_PURGE_RES)
                     2.16.840.1.113719.1.27.100.53 (TRIGGER_SCHEMA_SYNC_REQ)     
                     ...
******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ldap.h>
#include <ldap_oids.h>

static char usage[] =
"\n Usage:   getdse <host name> <port number> <login dn> <password>"
"\n Example: getdse Acme.com 389 cn=admin,o=Acme secret\n";


int main( int argc, char **argv) { 

    int         version, ldapPort, i, j, rc; 
    char        *ldapHost, *loginDN, *password;    
    char        *attrName ;
    char        **values;       
    struct      timeval timeOut;
    BerElement  *ber;
    LDAP        *ld;
    LDAPMessage *searchResult = NULL, *dse;    

    if (argc != 5) {
        printf("%s", usage);
        return(1);
    }

    ldapHost          = argv[1];
    ldapPort          = atoi(argv[2]);
    loginDN           = argv[3];
    password          = argv[4];   
    timeOut.tv_sec    = 10L;
    timeOut.tv_usec   = 0L;

    /* Set LDAP version to 3 */
    version = LDAP_VERSION3;
    ldap_set_option( NULL, LDAP_OPT_PROTOCOL_VERSION, &version);

    /* Initialize the LDAP session */
    if (( ld = ldap_init( ldapHost, ldapPort )) == NULL) {
        printf ( "\n    LDAP session initialization failed\n");
        return( 1 );
    }
    printf ( "\n    LDAP session initialized\n");

    /* Bind to the server */
    rc = ldap_simple_bind_s( ld, loginDN, password );
    if (rc != LDAP_SUCCESS ) {
        printf("ldap_simple_bind_s: %s\n", ldap_err2string( rc ));
        ldap_unbind_s ( ld );
        return( 1 );
    }
    printf("\n    Bind successful\n");

    /*
     * To search for rootDSE,
     *  1. Must be set for LDAP v3 before binding.
     *  2. Set search base = NULL or empty string
     *  3. Set search filter = (objectclass=*)
     *  4. Set search scope = LDAP_SCOPE_BASE
     */
    rc = ldap_search_ext_s(
                ld,                 /* LDAP session handle */
                NULL,               /* search base */
                LDAP_SCOPE_BASE,    /* scope */ 
                "(objectclass=*)",  /* search filter */
                NULL,               /* return all attrs */
                0,                  /* return both attrs and values */
                NULL,               /* server controls */
                NULL,               /* client controls */
                &timeOut,           /* time out */
                LDAP_NO_LIMIT,      /* no limit on entries */
                &searchResult );    /* returned search results */

    if ( rc != LDAP_SUCCESS ) {
        printf(  "ldap_search_ext_s: %s\n", ldap_err2string( rc ));          
        ldap_unbind_s( ld );
        return( 1 );
    }


    /* We have only one entry - root dse in search result */
    dse = ldap_first_entry( ld, searchResult );  

    /* print out all the attributes and attribute values */
    printf("\n    Root DSE");
    printf("\n    -----------------------------------------\n");
    for (   attrName  = ldap_first_attribute( ld, searchResult, &ber );
            attrName  != NULL; 
            attrName  = ldap_next_attribute( ld, searchResult, ber )) {
        if (( values = ldap_get_values( ld, dse, attrName )) != NULL ) {
            
			printf("    %s:\n", attrName );
            
			for ( i = 0; values[ i ] != NULL; i++ ) {
				
				if ( strcmp(attrName, "supportedExtension") == 0
                    || strcmp(attrName, "supportedControl") == 0) {
                    printf("        %s", values[i]);                        
                    for ( j = 0; j < TOTAL_LEN; j++ ) {
                        if ( strcmp( values[i], namesandoids[j][3]) == 0 ) {
                            if( strlen(namesandoids[j][1]) )
								printf("   (%s)\n", namesandoids[j][1]);								
							else 								
								printf("\n");

                            break;
						}                       
					}                    
                }
				
                else
					printf("        %s\n", values[ i ] );
			}
            ldap_value_free( values );
        }
        ldap_memfree( attrName  );
    }                     

    ber_free(ber, 0);
    ldap_msgfree( searchResult );
    ldap_unbind_s( ld );   

    return( 0 );
}

