/* $Novell: /ldap/src/cldap/samples/operationalAttrs.c,v 1.4 2001/08/23 22:58:07 dsteck Exp $ */
/*************************************************************************

   Copyright 2001 Novell, Inc.  All Rights Reserved.

   With respect to this file, Novell hereby grants to Developer a 
   royalty-free, non-exclusive license to include this sample code 
   and derivative binaries in its product. Novell grants to Developer 
   worldwide distribution rights to market, distribute or sell this 
   sample code file and derivative binaries as a component of 
   Developer's product(s).  Novell shall have no obligations to 
   Developer or Developer's customers with respect to this code.

   DISCLAIMER:

   Novell disclaims and excludes any and all express, implied, and 
   statutory warranties, including, without limitation, warranties 
   of good title, warranties against infringement, and the implied 
   warranties of merchantability and fitness for a particular purpose.  
   Novell does not warrant that the software will satisfy customer's 
   requirements or that the licensed works are without defect or error 
   or that the operation of the software will be uninterrupted.  
   Novell makes no warranties respecting any technical services or 
   support tools provided under the agreement, and disclaims all other 
   warranties, including the implied warranties of merchantability and 
   fitness for a particular purpose.

***************************************************************************
    operationalAttrs.c
***************************************************************************
   Description: operationalAttrs.c shows LDAP operational attributes 
                of an entry.
                
                LDAP operational attributes are not returned in search 
                results unless explicitly requested by name.
***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ldap.h>
#include <time.h>

static char usage[] =
"\nUsage:   operationalAttrs <host name> <port number> <login dn> <password>" 
"\n                          <entryDN>"
"\nExample: operationalAttrs Acme.com 389 cn=admin,o=Acme secret"
"\n                          cn=James,o=Acme\n";

int  printOperationalAttrs( LDAP* , char* );
void printTime( char * );
void displayEntryFlags( int );

#define DS_ALIAS_ENTRY           0x0001
#define DS_PARTITION_ROOT        0x0002
#define DS_CONTAINER_ENTRY       0x0004
#define DS_CONTAINER_ALIAS       0x0008
#define DS_MATCHES_LIST_FILTER   0x0010
#define DS_REFERENCE_ENTRY       0x0020
#define DS_40X_REFERENCE_ENTRY   0x0040
#define DS_BACKLINKED            0x0080
#define DS_NEW_ENTRY             0x0100
#define DS_TEMPORARY_REFERENCE   0x0200
#define DS_AUDITED               0x0400
#define DS_ENTRY_NOT_PRESENT     0x0800
#define DS_ENTRY_VERIFY_CTS      0x1000
#define DS_ENTRY_DAMAGED         0x2000

int main( int argc, char **argv) {
    
    int      rc, ldapPort, version;
    char     *ldapHost, *loginDN, *password, *testObjDN;
    LDAP     *ld;

    if (argc != 6) {
        printf("%s", usage);
        return (1);
    }

    ldapHost   =  argv[1];
    ldapPort   =  atoi(argv[2]);
    loginDN    =  argv[3];
    password   =  argv[4];
    testObjDN  =  argv[5];

    /* 
     * Set LDAP version to 3.
     * Using a NULL session handle sets the global options.
     * All subsequent sessions in this process will be version 3.
     */
    version = LDAP_VERSION3;
    ldap_set_option( NULL, LDAP_OPT_PROTOCOL_VERSION, &version);
    
    /* Initialize an LDAP session */
    if (( ld = ldap_init( ldapHost, ldapPort )) == NULL) {
        printf ( "\n    LDAP session initialization failed.\n");
        return( 1 );
    }
    printf ( "\n    New LDAP session initialized.\n");
    
    /* Simple bind to the server */
    rc = ldap_simple_bind_s( ld, loginDN, password );

    if ( rc == LDAP_SUCCESS ) {
        printf("\n    Bind successful.\n");
    }
    else {
        printf("\n    ldap_simple_bind_s: %s\n", ldap_err2string( rc ));
        ldap_unbind_s ( ld );
        return( 1 );
    }

    /* Get LDAP operational attributes of the entry */
    if ( (rc = printOperationalAttrs( ld, testObjDN )) != LDAP_SUCCESS )
        printf("\n    Unable to read entryDN object: %s\n", 
                                                  ldap_err2string( rc ));
    
    ldap_unbind_s( ld );

    return ( 0 );
}

/*
 * printOperationalAttrs() searches the directory to get LDAP 
 * operational attributes of the entry specified by dn, and then 
 * prints them out.
 */
int printOperationalAttrs( LDAP *ld, char * dn ) {
        
    int         rc, flags;    
    char        **values, *attribute;
    struct      timeval timeOut;
    BerElement  *ber;
    LDAPMessage *searchResult, *entry;
    char        *attrs[] = { "createTimeStamp",
                             "creatorsName",
                             "entryFlags",
                             "federationBoundary",
                             "localEntryID",
                             "modifiersName",
                             "modifyTimeStamp",
                             "structuralObjectClass",
                             "subordinateCount",
                             "subschemaSubentry",
                              NULL };

    timeOut.tv_sec    = 10L;
    timeOut.tv_usec   = 0L;    
        
    /* Get the entry's LDAP operational attributes */
    rc = ldap_search_ext_s(  
        ld,                    /* LDAP session handle */
        dn,                    /* object to read */
        LDAP_SCOPE_BASE,       /* search scope */
        "(objectclass=*)",     /* search filter */
        attrs,                 /* return the LDAP operational attributes */
        0,                     /* return both the attributes and the values */
        NULL,                  /* server controls */
        NULL,                  /* client controls */
        &timeOut,              /* time out */
        LDAP_NO_LIMIT,         /* no size limit */
        &searchResult );       /* returned results */

    if ( rc != LDAP_SUCCESS )        
        return ( rc );

    /* Print out the LDAP operational attributes and values */
    entry   =   ldap_first_entry( ld, searchResult );

    printf("\n    Entry: %s\n", dn);
    printf("\n    LDAP Operational Attributes: ");
    
    for (   attribute = ldap_first_attribute( ld, entry, &ber );
            attribute != NULL; 
            attribute = ldap_next_attribute( ld, entry, ber ) ) {
        
        if( (values = ldap_get_values( ld, entry, attribute)) != NULL ) {
            if ( (strcmp( attribute, "modifyTimeStamp" ) == 0) 
                                                && (values[0] != NULL)) {
                printf("\n        %s:", attribute );
                printf("\n                %s (UTC)", values[0] );
                printTime( values[0] );
            }
            else if ( (strcmp( attribute, "createTimeStamp" ) == 0)
                                                && (values[0] != NULL)) {
                printf("\n        %s:", attribute );
                printf("\n                %s (UTC)", values[0] );
                printTime( values[0] );
            }
            else if ( (strcmp( attribute, "entryFlags" ) == 0)
                                                && (values[0] != NULL)) {
                sscanf( values[0], "%d", &flags);
                if ( flags  != 0 )    
                    displayEntryFlags( flags );
            }
            else if ( values[0] != NULL )
                printf("\n        %s: %s", attribute, values[0] );
            
            ldap_value_free( values );
            ldap_memfree( attribute );
        }        
    }

    printf("\n\n");    
    ber_free(ber, 0);
    ldap_msgfree( searchResult );

    return rc;
}

/* displayEntryFlags() decodes and displays OR'ed entry flags */
void displayEntryFlags( int flags ) 
{    
    char entryFlags[128];    
                    
    /* Start from the beginning of the buffer */
    strcpy( entryFlags, "" );
        
    if ( flags & DS_ALIAS_ENTRY )
        strcat( entryFlags, "AliasEntry ");
    if ( flags & DS_PARTITION_ROOT )
        strcat( entryFlags, "PartionRoot ");
    if ( flags & DS_CONTAINER_ENTRY )
        strcat( entryFlags, "ContainerEntry ");
    if ( flags & DS_CONTAINER_ALIAS )
        strcat( entryFlags, "ContainerAlias ");
    if ( flags & DS_MATCHES_LIST_FILTER )
        strcat( entryFlags, "MatchesListFilter ");
    if ( flags & DS_REFERENCE_ENTRY )
        strcat( entryFlags, "ReferenceEntry ");
    if ( flags & DS_40X_REFERENCE_ENTRY )
        strcat( entryFlags, "40XReferenceEntry ");
    if ( flags & DS_BACKLINKED )
        strcat( entryFlags, "Backlinked ");
    if ( flags & DS_NEW_ENTRY )
        strcat( entryFlags, "NewEntry ");
    if ( flags & DS_TEMPORARY_REFERENCE )
        strcat( entryFlags, "TemporaryEntry ");
    if ( flags & DS_AUDITED )
        strcat( entryFlags, "Audited ");
    if ( flags & DS_ENTRY_NOT_PRESENT )
        strcat( entryFlags, "EntryNotPresent ");
    if ( flags & DS_ENTRY_VERIFY_CTS )
        strcat( entryFlags, "EntryVarifyCTS ");
    if ( flags & DS_ENTRY_DAMAGED )
        strcat( entryFlags, "EntryDamaged");
 
    printf("\n        entryFlags: %s", &entryFlags);
}

/* printTime() prints a Generalized Time string in a more readable format */
void printTime( char *utc ) {
    
    char   *time;
    struct tm UTC;

    /* Initialize UTC fields with year, mon, day, hour, min, and sec 
     * in utc. utc is in the form of "yyyymmddhhmmssZ"
     */
    sscanf( utc, "%4d%02d%02d%02d%02d%02d", &UTC.tm_year, &UTC.tm_mon,
        &UTC.tm_mday, &UTC.tm_hour, &UTC.tm_min, &UTC.tm_sec );

    /* Adjust year and month field */
    UTC.tm_year -= 1900;
    UTC.tm_mon  -= 1;
    UTC.tm_isdst = 0;

    if ( mktime(&UTC) != (time_t)-1 ) {
        time = asctime(&UTC);
        /* Remove trailing new line char */
        time[strlen(time)-1] = '\0';
        printf( "\n                %s (UTC)", time );
    }
}
