/* $Novell: /ldap/src/cldap/samples/searchBinary.c,v 1.4 2001/08/23 22:57:37 dsteck Exp $ */
 /*************************************************************************

   Copyright 2001 Novell, Inc.  All Rights Reserved.

   With respect to this file, Novell hereby grants to Developer a 
   royalty-free, non-exclusive license to include this sample code 
   and derivative binaries in its product. Novell grants to Developer 
   worldwide distribution rights to market, distribute or sell this 
   sample code file and derivative binaries as a component of 
   Developer's product(s).  Novell shall have no obligations to 
   Developer or Developer's customers with respect to this code.

   DISCLAIMER:

   Novell disclaims and excludes any and all express, implied, and 
   statutory warranties, including, without limitation, warranties 
   of good title, warranties against infringement, and the implied 
   warranties of merchantability and fitness for a particular purpose.  
   Novell does not warrant that the software will satisfy customer's 
   requirements or that the licensed works are without defect or error 
   or that the operation of the software will be uninterrupted.  
   Novell makes no warranties respecting any technical services or 
   support tools provided under the agreement, and disclaims all other 
   warranties, including the implied warranties of merchantability and 
   fitness for a particular purpose.

***************************************************************************
   searchBinary.c 
***************************************************************************
   Description: searchBinary.c is similar to search.c, but handles binary 
                values appropriately.  All values are read in binary form.  
                Any value with non-ASCII bytes is printed in hex, 
                while ASCII strings are printed normally.
***************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <ldap.h>

static char usage[] =
"\n Usage:   searchBinary <host name> <port number> <login dn> <password>"
"\n          <search base>\n" 
"\n Example: searchBinary Acme.com 389 cn=admin,o=Acme secret"
"\n          ou=Sales,o=Acme\n";

/* Prototypes for internal functions */
int  value_is_printable( struct berval *bval );
void binary_print( char *attr, struct berval *bval );


int main( int argc, char **argv )
{   
    int           version, ldapPort, i, rc;
    char          *ldapHost, *loginDN, *password, *searchBase;   
    char          *attribute, *dn, *sortAttribute;       
    struct        timeval timeOut;
    BerElement    *ber;
    LDAP          *ld;
    LDAPMessage   *searchResult, *entry;
    struct berval **bvalues;
    
   
    if (argc != 6)
    {
        printf("%s", usage);
        return (1);
    }
           
    ldapHost          = argv[1];
    ldapPort          = atoi(argv[2]);
    loginDN           = argv[3];
    password          = argv[4];
    searchBase        = argv[5];    

    /* Set LDAP version to 3 */
    version = LDAP_VERSION3;
    ldap_set_option( NULL, LDAP_OPT_PROTOCOL_VERSION, &version);

    /* Initialize the LDAP session */
    if (( ld = ldap_init( ldapHost, ldapPort )) == NULL)
    {
        printf ( "\n    LDAP session initialization failed\n");
        return( 1 );
    }
    printf ( "\n    LDAP session initialized\n");

    /* Bind to the server */
    rc = ldap_simple_bind_s( ld, loginDN, password );
    if (rc != LDAP_SUCCESS )
    {
        printf("ldap_simple_bind_s: %s\n", ldap_err2string( rc ));
        ldap_unbind_s ( ld );
        return( 1 );
    }
    printf("\n    Bind successful\n");


    timeOut.tv_sec    = 10L;               /* Set a 10 second timeout */
    timeOut.tv_usec   = 0L;
    
    /* Search the directory */
    rc = ldap_search_ext_s(  
                    ld,                    /* LDAP session handle */
                    searchBase,            /* container to search */
                    LDAP_SCOPE_ONELEVEL,   /* search scope */
                    "(objectclass=*)",     /* search filter */
                    NULL,                  /* return all attributes */
                    0,                     /* return attributes and values */
                    NULL,                  /* server controls */
                    NULL,                  /* client controls */
                    &timeOut,              /* time limit */
                    LDAP_NO_LIMIT,         /* no size limit */
                    &searchResult );       /* returned results */
    
    if ( rc != LDAP_SUCCESS ) 
    {
        printf("ldap_search_ext_s: %s\n", ldap_err2string( rc ));
        ldap_unbind_s( ld );
        return ( 1 );
    }   
     
    /* Sort the entries by surname (client-side sort) */
    sortAttribute = "sn";
    ldap_sort_entries( ld,                  /* LDAP session handle */
                       &searchResult,       /* results to sort */
                       sortAttribute,       /* attribute on which to sort */
                       strcmp );            /* comparison function to use */

    /* Go through the search results by checking entries */
    for (   entry   =   ldap_first_entry( ld, searchResult ); 
            entry   !=  NULL; 
            entry   =   ldap_next_entry( ld, entry ) ) 
    {
        if (( dn = ldap_get_dn( ld, entry )) != NULL )            
        {
            printf("\n    dn: %s\n", dn );
            ldap_memfree( dn );
        }
            
        for (   attribute = ldap_first_attribute( ld, entry, &ber );
                attribute != NULL; 
                attribute = ldap_next_attribute( ld, entry, ber ) ) 
        {   
            /* Get array of pointers to berval structures */
            if ((bvalues = ldap_get_values_len( ld, entry, attribute)) != NULL) 
            {
                for ( i = 0; bvalues[i] != NULL; i++ ) 
                {
                    if (value_is_printable( bvalues[i] ) )
                        printf("        %s: %s\n", 
                               attribute, bvalues[i]->bv_val );
                    else
                        binary_print( attribute, bvalues[i] );
                }
                ldap_value_free_len( bvalues );
            }
            ldap_memfree( attribute );
        }
       
        ber_free(ber, 0);

    }

    printf("\n    Search completed successfully.\n");
   
    ldap_msgfree( searchResult );
    
    ldap_unbind_s( ld );  
 
    return( 0 );
}


/* 
   Return 1 if all bytes in the value are printable, otherwise return 0. 
*/
int  value_is_printable( struct berval *bval )
{
    unsigned int i;
    for ( i=0; i<bval->bv_len; i++)
    {
        if ( !isprint( (bval->bv_val)[i] ) )
            return 0;   /* At least one non-printable byte in value */
    }

    return 1;   /* All bytes are printable. */
}


/* 
   Print the binary value in hex 
*/
#define COLS 16                     /* Number of columns per row */ 
void binary_print( char *attr, struct berval *bval )
{
    int rows, final, r, i;

    rows = bval->bv_len / COLS;     /* Number of full rows */
    final = bval->bv_len % COLS;    /* Number of values on last partial row */

    printf("        %s:\n", attr );
    
    for ( r=0; r<rows; r++ )
    {
        printf("           ");
        for ( i=0; i<COLS; i++ )
            printf(" %02X", (unsigned char)bval->bv_val[r*COLS+i]);
        printf("\n");
    }

    if (final)      /* Print remaining partial line, if any. */
    {
        printf("           ");
        for (i=0; i<final; i++)
            printf(" %02X", (unsigned char)bval->bv_val[rows*COLS+i]);
        printf("\n");
    }
}

