/* $Novell: /ldap/src/cldap/samples/searchmsg.c,v 1.5 2001/04/03 18:05:21 dsteck Exp $ */
/*************************************************************************

   Copyright 1999, 2000, 2001 Novell, Inc.  All Rights Reserved.

   With respect to this file, Novell hereby grants to Developer a 
   royalty-free, non-exclusive license to include this sample code 
   and derivative binaries in its product. Novell grants to Developer 
   worldwide distribution rights to market, distribute or sell this 
   sample code file and derivative binaries as a component of 
   Developer's product(s).  Novell shall have no obligations to 
   Developer or Developer's customers with respect to this code.

   DISCLAIMER:

   Novell disclaims and excludes any and all express, implied, and 
   statutory warranties, including, without limitation, warranties 
   of good title, warranties against infringement, and the implied 
   warranties of merchantability and fitness for a particular purpose.  
   Novell does not warrant that the software will satisfy customer's 
   requirements or that the licensed works are without defect or error 
   or that the operation of the software will be uninterrupted.  
   Novell makes no warranties respecting any technical services or 
   support tools provided under the agreement, and disclaims all other 
   warranties, including the implied warranties of merchantability and 
   fitness for a particular purpose.

***************************************************************************
   searchMsg.c
***************************************************************************
   Description: The searchMsg.c sample performs a search and parses 
                all search results, not just entries.  This sample
                will cause referrals to be returned and printed
                instead of automatically following them. 
                
                Note:  For simplicity, this sample prints all values as 
                if they were strings.  See the searchBinary.c sample program
                for an example of handling both string and binary values.
***************************************************************************/ 

#include <stdio.h>
#include <stdlib.h>
#include <ldap.h>

static char usage[] =
"\n Usage:   searchMsg <host name> <port number> <login dn> <password>"
"\n          <search base>\n" 
"\n Example: searchMsg acme.com 389 cn=admin,o=acme secret"
"\n          ou=Sales,o=Acme\n\n";

int main( int argc, char **argv)
{ 

    int         version, ldapPort, i, rc, messageID, parseResult;
    int         messageType, entryCount, referenceCount, messageCount;  
    char        *ldapHost, *loginDN, *password, *searchBase;    
    char        *attribute, *dn, **values, **referrals;       
    struct      timeval timeOut;
    LDAP        *ld;
    BerElement  *ber;
    LDAPMessage *searchResult, *message;


    if (argc != 6)
    {
        printf("%s", usage);
        return(1);
    }

    ldapHost          =   argv[1];
    ldapPort          =   atoi(argv[2]);
    loginDN           =   argv[3];
    password          =   argv[4];
    searchBase        =   argv[5];
    timeOut.tv_sec    =   10L;
    timeOut.tv_usec   =   0L;


    /* Set LDAP version to v3 */
    version = LDAP_VERSION3;
    ldap_set_option( NULL, LDAP_OPT_PROTOCOL_VERSION, &version );

    /* Don't automatically follow referrals.  Return them.
       Note that the server must also be set up to return referrals. */
    ldap_set_option( NULL, LDAP_OPT_REFERRALS, LDAP_OPT_OFF );

    /* Initialize an LDAP session */
    if (( ld = ldap_init( ldapHost, ldapPort )) == NULL)
    {
        printf("\n\tLDAP session initialization failed\n");
        return( 1 );
    }

    /* Perform a simple bind to the server */
    rc = ldap_simple_bind_s( ld, loginDN, password );
    if (rc != LDAP_SUCCESS )
    {
        printf("ldap_simple_bind_s: %s\n", ldap_err2string( rc ));
        ldap_unbind_s( ld );
        return( 1 );
    }

    /* Search the directory */
    rc = ldap_search_ext_s(
                ld,                  /* LDAP session handle */ 
                searchBase,          /* container to search */
                LDAP_SCOPE_SUBTREE,  /* search entire subtree */
                "(objectclass=*)",   /* search filter */
                NULL,                /* return all attributes */
                0,                   /* return both attributes and values */
                NULL,                /* search controls */
                NULL,                /* client controls */
                &timeOut,            /* time out */ 
                LDAP_NO_LIMIT,       /* no size limit */
                &searchResult );     /* returned results */

    if ( rc != LDAP_SUCCESS )
    {
        printf("ldap_search_ext_s: %s\n", ldap_err2string( rc ));
        ldap_unbind_s( ld );
        return( 1 );
    }

    /* Go through the search results by checking message types */
    for (   message = ldap_first_message( ld, searchResult ); 
        message != NULL; 
        message = ldap_next_message( ld, message ) )
    {
        messageType = ldap_msgtype( message );         
        printf("\nMessage type: %d\n", messageType );

        messageID = ldap_msgid( message );        
        printf("Message ID: %d\n", messageID );

        switch ( messageType )
        {
            /*
             * For an entry, print out its DN, attributes,
             * and attribute values  
             */
            case LDAP_RES_SEARCH_ENTRY:                 
                if (( dn = ldap_get_dn( ld, message )) != NULL )
                {
                    printf("\tdn: %s\n", dn );
                    ldap_memfree( dn );
                }

                for ( attribute = ldap_first_attribute( ld,searchResult,&ber );
                    attribute != NULL; 
                    attribute = ldap_next_attribute( ld, searchResult,ber))
                {
                    /* Get values and print.  Assumes all values are strings. */
                    if (( values = ldap_get_values( ld, message, attribute)) 
                        != NULL )
                    {
                        for ( i = 0; values[ i ] != NULL; i++ )
                            printf("\t\t%s: %s\n", attribute, values[ i ] );

                        ldap_value_free( values );
                    }

                    ldap_memfree( attribute );
                }

                ber_free(ber, 0);
                break;

            /* Print out all the URLs */
            case LDAP_RES_SEARCH_REFERENCE:                   
                parseResult = ldap_parse_reference(
                                    ld,             /* LDAP session handle */
                                    message,        /* message to be parsed */
                                    &referrals,     /* where to put referrals */
                                    NULL,           /* server controls */
                                    0);             /* not free the resource */ 
                
                if ( parseResult != LDAP_SUCCESS )
                {
                    printf("ldap_parse_result: %s\n",
                           ldap_err2string( parseResult ) );
                    ldap_msgfree( searchResult );
                    ldap_unbind_s( ld );
                    return( 1 );
                }

                if ( referrals != NULL )
                {
                    for (   i = 0; referrals[ i ] != NULL; i++ )
                        printf("Search reference: %s\n\n", referrals[ i ] );
                    ldap_value_free( referrals );
                }
                break; 

            /* This is the last message be returned */
            case LDAP_RES_SEARCH_RESULT:       
                break;

            default:
                break;
        }            

    }
        
    /* Statistics of returned message(s) */
    messageCount   = ldap_count_messages( ld, searchResult );
    entryCount     = ldap_count_entries( ld, searchResult );
    referenceCount = ldap_count_references( ld, searchResult ); 

    printf("\nSearch completed successfully.\n"
           "\nMessages returned: %d\n"
           "Entries  returned: %d\n"
           "Search references returned: %d\n",
           messageCount, entryCount, referenceCount );

    ldap_msgfree( searchResult );
    ldap_unbind_s( ld );   

    return( 0 );
}
