/* $Novell: /ldap/src/cldap/samples/sortcntl.c,v 1.5 2001/07/30 18:43:17 dsteck Exp $ */
/*************************************************************************

   Copyright 1999, 2000 Novell, Inc.  All Rights Reserved.

   With respect to this file, Novell hereby grants to Developer a 
   royalty-free, non-exclusive license to include this sample code 
   and derivative binaries in its product. Novell grants to Developer 
   worldwide distribution rights to market, distribute or sell this 
   sample code file and derivative binaries as a component of 
   Developer's product(s).  Novell shall have no obligations to 
   Developer or Developer's customers with respect to this code.

   DISCLAIMER:

   Novell disclaims and excludes any and all express, implied, and 
   statutory warranties, including, without limitation, warranties 
   of good title, warranties against infringement, and the implied 
   warranties of merchantability and fitness for a particular purpose.  
   Novell does not warrant that the software will satisfy customer's 
   requirements or that the licensed works are without defect or error 
   or that the operation of the software will be uninterrupted.  
   Novell makes no warranties respecting any technical services or 
   support tools provided under the agreement, and disclaims all other 
   warranties, including the implied warranties of merchantability and 
   fitness for a particular purpose.

***************************************************************************
   sortcntl.c
***************************************************************************
   Description: The sorcntl.c sample sets up a server-side sort control, 
                searches the directory, and prints the entry name, sn, 
                and given name.
                
                This sample also demonstrates how to return only 
                specified attributes.
***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <ldap.h>

static char usage[] =
"\n Usage:   sortcntl <host name> <port number> <login dn> <password>"
"\n\t  <search base>\n" 
"\n Example: sortcntl acme.com 389 cn=admin,o=acme secret ou=Sales,o=acme\n";

int main( int argc, char **argv)
{
    int   ldapPort;
    char  *ldapHost, *loginDN, *password, *searchBase;
    LDAP  *ld;
    int   rc;
    int   version;
    int   err;
    unsigned long err1;
    char  *dn = NULL;
    char  **values = NULL;
    struct      timeval timeOut;
    LDAPSortKey **sortkeylist = NULL;
    LDAPMessage *searchResult = NULL;
    LDAPMessage *entry = NULL;
    LDAPControl *serverctrls[2] = {NULL, NULL};    /* Ensure null-terminated */
    LDAPControl **resultControls = NULL;

    /*
     * Search parameters.
     */
    char  *attrs[] = {"givenname","sn", NULL };     /* Attributes to return */       
    char  *filter = "(objectclass=inetorgperson)";  /* Search filter */
    int   attrsonly = 0;                            /* Return attrs and values*/
    
    /*
     * Sort control parameters.
     */
    char  *sortAttr = "sn";                        /* Attribute to sort on */
    int   isCritical = 1;                          /* Control is critical */
    
    if (argc != 6)
    {
        printf("%s", usage);
        return ( 1 );
    }
    
    ldapHost   = argv[1];
    ldapPort   = atoi(argv[2]);
    loginDN    = argv[3];
    password   = argv[4];
    searchBase = argv[5];
    timeOut.tv_sec    =   10L;
    timeOut.tv_usec   =   0L;
    
    printf("\nSort key:  %s\n", sortAttr);
     if (isCritical)
        printf("Control is CRITICAL\n\n");
     else
        printf("Control is NOT critical\n\n");
    
    
    /* 
     * Set LDAP version 3 protocol 
     */
    version = LDAP_VERSION3;
    ldap_set_option( NULL, LDAP_OPT_PROTOCOL_VERSION, &version);

    /* 
     * Initialize an LDAP session
     */
    ld = ldap_init( ldapHost, ldapPort );
    if (NULL == ld)
    {
        printf("\n\tLDAP session initialization failed\n");
        return ( 1 );
    }        
    
    /* 
     * Authenticate to the server 
     */
    rc = ldap_simple_bind_s( ld, loginDN, password );
    if (LDAP_SUCCESS != rc)
    {
        printf("\n\tldap_simple_bind_s: %s\n", ldap_err2string(rc));
        ldap_unbind_s( ld );
        return ( rc );
    }
        
    /* 
     * Create the sort key structure specifying the attribute to sort on. 
     */
    rc = ldap_create_sort_keylist(&sortkeylist, sortAttr);
    if (LDAP_SUCCESS != rc)
    {
        printf("\n\tldap_create_sort_keylist: %s\n", ldap_err2string(rc));
        goto cleanup;
    }

    /* 
     * Create the sort control. 
     */
    rc = ldap_create_sort_control(
            ld,                         /* Session handle */
            sortkeylist,                /* Array of sortkey structs */
            isCritical,                 /* Criticality of this control */
            &serverctrls[0]);           /* The returned control structure */                        
    if (LDAP_SUCCESS != rc)
    {
        printf("\n\tldap_create_sort_control: %s\n",ldap_err2string(rc));
        goto cleanup;
    }
    
    /*
     * Perform the search with server-side sorting.
     */
    rc = ldap_search_ext_s( 
            ld,                         /* Session handle */
            searchBase,                 /* Container to search */
            LDAP_SCOPE_ONELEVEL,        /* Search this container only */
            filter,                     /* Search filter to use */
            attrs,                      /* Attributes to return */
            attrsonly,                  /* Return just attr names? */
            serverctrls,                /* Server controls */
            NULL,                       /* No client controls */
            &timeOut,                   /* time out */
            LDAP_NO_LIMIT,              /* No size limit */ 
            &searchResult);             /* Returned results */

    if (LDAP_SUCCESS != rc)
    {
        printf("\n\tldap_search_ext_s: %s\n", ldap_err2string(rc));
        goto cleanup;
    }

    /*
     * Parse the search results to obtain the return controls.
     */
    rc = ldap_parse_result(
            ld,                         /* Session handle */
            searchResult,               /* Results from the search operation */
            &err,                       /* Error code of the search operation */
            NULL,                       /* Matched DN */
            NULL,                       /* Error message */
            NULL,                       /* Referrals */ 
            &resultControls,            /* Returned controls */
            0);                         /* Don't free the searchResult */
    
    if (LDAP_SUCCESS != rc)
    {
        printf("\n\tldap_parse_result: %s\n", ldap_err2string(rc));
        goto cleanup;
    }

    /* 
     * Parse the returned sort control.
     */
    rc = ldap_parse_sort_control(
            ld,                     /* Session handle */                   
            resultControls,         /* Results from the search operation */
            &err1,                   /* Error code for the vlv control */   
            NULL);                  /* Attribute involved in the error */  

    if (LDAP_SUCCESS != rc)
    {
        printf("\n\tldap_parse_sort_control: %s\n", ldap_err2string(rc));
        goto cleanup;
    }
    if (err)
    {
        /* 
         * If sort control is not critical, the search continues.
         */
        printf("*** Sort control error: %s\n\n", ldap_err2string((int)err));
        printf("    The control was ignored.\n");
    }


    /* 
     * For each entry, print out dn, sn, and givenname.
     * Only print first value if there are multiple values.
     */
    printf("Entry <dn>:   <sn>, <given name>\n\n");
    for ( entry = ldap_first_entry( ld, searchResult ); 
          entry != NULL;
          entry = ldap_next_entry( ld, entry ) ) 
    {
        dn = ldap_get_dn( ld, entry );
        if (dn)
        {
            printf("Entry %s:\t",dn);
            ldap_memfree(dn);
        }

        if ((values = ldap_get_values( ld, entry, "sn")) != NULL ) 
        {
            if ( values[0] != NULL )    
                printf("%s,  ", values[0] );
            ldap_value_free( values );
        }
   
        if ((values = ldap_get_values( ld, entry, "givenname")) != NULL ) 
        {
           if ( values[0] != NULL ) 
               printf("%s", values[0] );
           ldap_value_free( values );
        }
        printf("\n" );
    }
            
cleanup:
    ldap_msgfree( searchResult );
    ldap_control_free( serverctrls[0] );
    ldap_controls_free( resultControls );
    ldap_free_sort_keylist( sortkeylist );
    ldap_unbind_s( ld );
   
    return( rc );
}
